

import * as List from "bs-platform/lib/es6/list.js";
import * as $$Array from "bs-platform/lib/es6/array.js";
import * as Block from "bs-platform/lib/es6/block.js";
import * as Format from "bs-platform/lib/es6/format.js";
import * as $$String from "bs-platform/lib/es6/string.js";
import * as Pervasives from "bs-platform/lib/es6/pervasives.js";
import * as Caml_format from "bs-platform/lib/es6/caml_format.js";
import * as Caml_option from "bs-platform/lib/es6/caml_option.js";
import * as Caml_exceptions from "bs-platform/lib/es6/caml_exceptions.js";
import * as Caml_js_exceptions from "bs-platform/lib/es6/caml_js_exceptions.js";
import * as Caml_builtin_exceptions from "bs-platform/lib/es6/caml_builtin_exceptions.js";

var WrongPlatform = Caml_exceptions.create("AST.WrongPlatform");

function unavailable_on(target, name) {
  throw [
        WrongPlatform,
        target,
        name
      ];
}

function tOfJs(param) {
  throw [
        WrongPlatform,
        /* Native */-693027401,
        "tOfJs"
      ];
}

function expressionOfJs(param) {
  throw [
        WrongPlatform,
        /* Native */-693027401,
        "expressionOfJs"
      ];
}

function to_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "to_yojson"
      ];
}

function of_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "of_yojson"
      ];
}

function expression_to_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_to_yojson"
      ];
}

function expression_of_yojson(param) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_of_yojson"
      ];
}

function flagToJs(param) {
  return {
          name: param[/* name */0],
          payload: param[/* payload */1]
        };
}

function flagFromJs(param) {
  return /* record */[
          /* name */param.name,
          /* payload */param.payload
        ];
}

function expressionToJs(param) {
  return {
          count: param[/* count */0],
          cmd: param[/* cmd */1],
          rev_args: param[/* rev_args */2]
        };
}

function expressionFromJs(param) {
  return /* record */[
          /* count */param.count,
          /* cmd */param.cmd,
          /* rev_args */param.rev_args
        ];
}

function tToJs(param) {
  return {
          expressions: param[/* expressions */0]
        };
}

function tFromJs(param) {
  return /* record */[/* expressions */param.expressions];
}

function make_expression(count, cmd, rev_args) {
  return /* record */[
          /* count */count !== undefined ? Caml_format.caml_int_of_string(count) : 1,
          /* cmd */cmd,
          /* rev_args */rev_args
        ];
}

function pipe_last(from, into) {
  var match = into[/* rev_args */2];
  var new_args;
  if (match) {
    var match$1 = match[0];
    if (match$1.tag) {
      var flg = match$1[0];
      var match$2 = flg[/* payload */1];
      if (typeof match$2 === "number") {
        if (match$2 !== 0) {
          var new_flag = /* record */[
            /* name */flg[/* name */0],
            /* payload : Unresolved */0
          ];
          new_args = /* :: */[
            /* Flag */Block.__(1, [new_flag]),
            match[1]
          ];
        } else {
          new_args = Pervasives.failwith("unreachable: pipe_last, unresolved-flag in last position");
        }
      } else {
        new_args = into[/* rev_args */2];
      }
    } else {
      new_args = into[/* rev_args */2];
    }
  } else {
    new_args = /* [] */0;
  }
  var new_args_000 = /* Positional */Block.__(0, [/* Sub */Block.__(0, [from])]);
  var new_args$1 = /* :: */[
    new_args_000,
    new_args
  ];
  return /* record */[
          /* count */into[/* count */0],
          /* cmd */into[/* cmd */1],
          /* rev_args */new_args$1
        ];
}

function copy_string_or_subexpr(sos) {
  if (sos.tag) {
    return /* Literal */Block.__(1, [$$String.copy(sos[0])]);
  } else {
    return /* Sub */Block.__(0, [copy_expression(sos[0])]);
  }
}

function copy_flag(flg) {
  var match = flg[/* payload */1];
  return /* record */[
          /* name */flg[/* name */0],
          /* payload */typeof match === "number" ? (
              match !== 0 ? /* Absent */1 : /* Unresolved */0
            ) : /* Resolved */[copy_string_or_subexpr(match[0])]
        ];
}

function copy_arg(arg) {
  if (arg.tag) {
    return /* Flag */Block.__(1, [copy_flag(arg[0])]);
  } else {
    return /* Positional */Block.__(0, [copy_string_or_subexpr(arg[0])]);
  }
}

function copy_expression(expr) {
  return /* record */[
          /* count */expr[/* count */0],
          /* cmd */copy_string_or_subexpr(expr[/* cmd */1]),
          /* rev_args */List.map(copy_arg, expr[/* rev_args */2])
        ];
}

function copy(ast) {
  return /* record */[/* expressions */$$Array.map(copy_expression, ast[/* expressions */0])];
}

function pp_bs(ast) {
  var obj = tToJs(ast);
  console.log(Caml_option.undefined_to_opt(JSON.stringify(obj)));
  return /* () */0;
}

function pp_native(ast) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "to_yojson"
      ];
}

function pp(ast) {
  try {
    return pp_bs(ast);
  }
  catch (raw_exn){
    var exn = Caml_js_exceptions.internalToOCamlException(raw_exn);
    if (exn[0] === WrongPlatform) {
      if (exn[1] !== -693027401) {
        throw exn;
      }
      return pp_native(ast);
    } else {
      throw exn;
    }
  }
}

function pp_expression_bs(expr) {
  var obj = expressionToJs(expr);
  console.log(Caml_option.undefined_to_opt(JSON.stringify(obj)));
  return /* () */0;
}

function pp_expression_native(expr) {
  throw [
        WrongPlatform,
        /* JavaScript */500616045,
        "expression_to_yojson"
      ];
}

function pp_expression(ast) {
  try {
    return pp_expression_bs(ast);
  }
  catch (raw_exn){
    var exn = Caml_js_exceptions.internalToOCamlException(raw_exn);
    if (exn[0] === WrongPlatform) {
      if (exn[1] !== -693027401) {
        throw exn;
      }
      return pp_expression_native(ast);
    } else {
      throw exn;
    }
  }
}

function is_literal(param) {
  if (param.tag) {
    return true;
  } else {
    return false;
  }
}

function get_literal_exn(param) {
  if (param.tag) {
    return param[0];
  } else {
    throw Caml_builtin_exceptions.not_found;
  }
}

function get_sub_exn(param) {
  if (param.tag) {
    throw Caml_builtin_exceptions.not_found;
  } else {
    return param[0];
  }
}

export {
  WrongPlatform ,
  unavailable_on ,
  tOfJs ,
  expressionOfJs ,
  to_yojson ,
  of_yojson ,
  expression_to_yojson ,
  expression_of_yojson ,
  flagToJs ,
  flagFromJs ,
  expressionToJs ,
  expressionFromJs ,
  tToJs ,
  tFromJs ,
  make_expression ,
  pipe_last ,
  copy_string_or_subexpr ,
  copy_flag ,
  copy_arg ,
  copy_expression ,
  copy ,
  pp_bs ,
  pp_native ,
  pp ,
  pp_expression_bs ,
  pp_expression_native ,
  pp_expression ,
  is_literal ,
  get_literal_exn ,
  get_sub_exn ,
  
}
/* Format Not a pure module */
